@@ -9,6 +9,8 @@ import torch
 import numpy as np
 import random
 
+from pathlib import Path
+import yaml
 from picsellia.sdk.asset import MultiAsset
 
 from picsellia.types.enums import LogType
@@ -134,6 +136,28 @@ def edit_model_yaml(label_map, experiment_name, config_path=None):
 
     return tmp
 
+
+def update_model_hyperparameters_config_file(hyperparameter_file_path: str, experiment_parameters: dict) -> None:
+    """ 
+    Looking for matching key between base configuration model parameters and experiment params
+    """
+    if not os.path.isfile(hyperparameter_file_path):
+        raise FileNotFoundError(f"Expected a .yaml file, got None at path: {hyperparameter_file_path}")
+    
+    model_parameters = yaml.safe_load(Path(hyperparameter_file_path).read_text())
+
+    for key, value in model_parameters.items():
+        if key in experiment_parameters:
+            if type(experiment_parameters[key]) not in [int, float]:
+                raise TypeError(f"Invalid type for parameter {key}, got {experiment_parameters[key]} expected {type(value)}")
+            model_parameters[key] = float(experiment_parameters[key])
+            print(f"Updating param: {key} to value: {value}")
+            
+    with open(hyperparameter_file_path, 'w+') as f:
+        yaml.dump(model_parameters, f, allow_unicode=True)
+    return
+
+
 def train_test_val_split(experiment, dataset, prop):
     train_assets, test_assets, train_split, test_split, labels = dataset.train_test_split(prop=prop, random_seed=42)
     
@@ -240,6 +264,7 @@ def setup_hyp(experiment=None, data_yaml_path=None, config_path= None, params={}
         if f.endswith('.yaml'):
             hyp_path = os.path.join(experiment.checkpoint_dir, f)
     
+    update_model_hyperparameters_config_file(hyperparameter_file_path=hyp_path, experiment_parameters=params)
     opt = Opt()
     opt.cwd = cwd
     opt.weights = weight_path 
@@ -259,7 +284,7 @@ def setup_hyp(experiment=None, data_yaml_path=None, config_path= None, params={}
     opt.bucket = ''
     opt.cache = 'ram'
     opt.image_weights = False
-    opt.device = 'cuda:0' if torch.cuda.is_available() else 'cpu'
+    opt.device = '0' if torch.cuda.is_available() else 'cpu'
     opt.multi_scale = True
     opt.single_cls = (len(label_map) == 1)
     opt.optimizer = 'Adam'
@@ -344,9 +369,18 @@ def get_metrics_curves(final_run_path):
     confusion_matrix = None
     F1_curve = None
     labels_correlogram = None
+    labels = None
     P_curve = None
     PR_curve = None
     R_curve = None
+    BoxF1_curve = None
+    BoxP_curve = None
+    BoxPR_curve = None
+    BoxR_curve = None
+    MaskF1_curve = None
+    MaskP_curve = None
+    MaskPR_curve = None
+    MaskR_curve = None
     if os.path.isfile(os.path.join(final_run_path, 'confusion_matrix.png')):
         confusion_matrix = os.path.join(final_run_path, 'confusion_matrix.png')
     if os.path.isfile(os.path.join(final_run_path, 'F1_curve.png')):
@@ -361,7 +395,23 @@ def get_metrics_curves(final_run_path):
         PR_curve = os.path.join(final_run_path, 'PR_curve.png')
     if os.path.isfile(os.path.join(final_run_path, 'R_curve.png')):
         R_curve = os.path.join(final_run_path, 'R_curve.png')
-    return confusion_matrix, F1_curve, labels_correlogram, labels, P_curve, PR_curve, R_curve
+    if os.path.isfile(os.path.join(final_run_path, 'BoxF1_curve.png')):
+        BoxF1_curve = os.path.join(final_run_path, 'BoxF1_curve.png')
+    if os.path.isfile(os.path.join(final_run_path, 'BoxP_curve.png')):
+        BoxP_curve = os.path.join(final_run_path, 'BoxP_curve.png')
+    if os.path.isfile(os.path.join(final_run_path, 'BoxPR_curve.png')):
+        BoxPR_curve = os.path.join(final_run_path, 'BoxPR_curve.png')
+    if os.path.isfile(os.path.join(final_run_path, 'BoxR_curve.png')):
+        BoxR_curve = os.path.join(final_run_path, 'BoxR_curve.png')
+    if os.path.isfile(os.path.join(final_run_path, 'MaskF1_curve.png')):
+        MaskF1_curve = os.path.join(final_run_path, 'MaskF1_curve.png')
+    if os.path.isfile(os.path.join(final_run_path, 'MaskP_curve.png')):
+        MaskP_curve = os.path.join(final_run_path, 'MaskP_curve.png')
+    if os.path.isfile(os.path.join(final_run_path, 'MaskPR_curve.png')):
+        MaskPR_curve = os.path.join(final_run_path, 'MaskPR_curve.png')
+    if os.path.isfile(os.path.join(final_run_path, 'MaskR_curve.png')):
+        MaskR_curve = os.path.join(final_run_path, 'MaskR_curve.png')
+    return confusion_matrix, F1_curve, labels_correlogram, labels, P_curve, PR_curve, R_curve, BoxF1_curve, BoxP_curve, BoxPR_curve, BoxR_curve, MaskF1_curve, MaskP_curve, MaskPR_curve, MaskR_curve
 
 def send_run_to_picsellia(experiment, cwd):
     final_run_path = find_final_run(cwd)
